/*
  All scheduleMap managed by schedule plugin

  mapId: div map ID
  map: map
  bounds: all POI position to compute center and zoom
  markersSize: number of POI
  markers: POI markers
*/
var scheduleMapList = new Array ();
/* POI markers icon URL */
var scheduleMarkerUrl;
/* empty icon URL */
var scheduleEmptyUrl;
var iconDim;
var iconOffset;
/* selected LI on city UL */
var scheduleSelectedCity = null;


// ========================================
/* center scheduleMap according POI markers */
function scheduleCenterMap (scheduleMap) {
    if (scheduleMap.markersSize) {
	scheduleMap.map.setCenter (scheduleMap.bounds.getCenterLonLat (),
				   Math.min (scheduleMap.map.getZoomForExtent (scheduleMap.bounds), 13));
	return;
    }
    // default : Mobihan
    scheduleMap.map.setCenter (new OpenLayers.LonLat (-2.671, 47.934).transform (
	new OpenLayers.Projection ('EPSG:4326'),
	new OpenLayers.Projection ('EPSG:900913')), 7);
}

// ========================================
/* remove all POI markers */
function scheduleClearMarkers (scheduleMap) {
    scheduleMap.markersSize = 0;
    scheduleMap.bounds = new OpenLayers.Bounds ();
    scheduleMap.markers.clearMarkers ();
}

// ========================================
/* add a positioned city marker */
function scheduleAddLatLonCityMarker (scheduleMap, cityCode, lat, lon, manageClick) {
    var lonlat = new OpenLayers.LonLat (lon, lat).transform (
	new OpenLayers.Projection ('EPSG:4326'),
	new OpenLayers.Projection ('EPSG:900913')
    );
    scheduleMap.bounds.extend (lonlat);
    scheduleMap.markersSize++;

    var emptyIcon = new OpenLayers.Icon (scheduleEmptyUrl, iconDim, iconOffset);

    var scheduleMarker = new OpenLayers.Marker (lonlat, emptyIcon);
    var iconDiv = jQuery (scheduleMarker.icon.imageDiv);
    iconDiv.addClass ('poiPin poiPinPush');
    iconDiv.attr ('city', cityCode);
    scheduleMarker.events.register ('mouseover', scheduleMarker, function (e) {
    	scheduleHighlightDays (cityCode);
    	scheduleHighlightAddress (cityCode);
    });
    scheduleMarker.events.register ('mouseout', scheduleMarker, function (e) {
    	scheduleHighlightDays (null);
    	scheduleHighlightAddress (null);
    });
    if (manageClick) {
    	scheduleMarker.events.register ('click', scheduleMarker, function (e) {
    	    window.open ("https://www.openstreetmap.org/?mlat="+lat+"&mlon="+lon+"&zoom=14", "_self");
    	});
    }
    scheduleMap.markers.addMarker (scheduleMarker);
}

// ========================================
/* highlight all cities on the list of adress */
function scheduleHighlightAddress (cities) {
    if (!cities) {
	jQuery ('div.scheduleAddresse').removeClass("poiAdrLight");
	return;
    }
    jQuery ('div.scheduleAddresse').each (function () {
	var showPOI = jQuery (this);
	var city = showPOI.attr ('city');
	if (city && (','+cities+',').indexOf (city)>=0)
	    showPOI.addClass("poiAdrLight");
	else
	    showPOI.removeClass("poiAdrLight");
    });
    return;
}

function scheduleHighlightAddressPOI (cities) {
    scheduleHighlightAddress (cities);
    scheduleHighlightPOI (cities);
}

// ========================================
/* add a wellknown city marker */
function scheduleAddCityMarker (scheduleMap, cityCode) {
    if (!(cityCode > 0))
	return;
    scheduleAddLatLonCityMarker (scheduleMap, cityCode,
				 scheduleCitiesInseeNameLatLon[cityCode][1],
				 scheduleCitiesInseeNameLatLon[cityCode][2],
				 true);
}

// ========================================
/* display cities from a UL */
function scheduleAddAllCityFromUl (scheduleMap, ul) {
    scheduleClearMarkers (scheduleMap);
    ul.find ('li').each (function (index) {
	var li = jQuery (this);
	scheduleAddLatLonCityMarker (scheduleMap, li.attr ('insee'),
				     li.attr ('lat'), li.attr ('lon'),
				     false);
    });
    scheduleMap.markers.redraw ();
}

// ========================================
function scheduleAddCityToUl (ul, cityName) {
    var cityCode = /[0-9]+/.exec (cityName);
    var lat = "";
    var lon = "";
    if (cityCode != null)
	cityCode = cityCode[0];
    if (!cityCode)
	cityCode = scheduleCitiesNameInsee [cityName.toLowerCase ()];

    if (!cityCode)
	cityName = cityCode = cityName.replace(/[^A-ZÀÂÆÇÉÈÊËÎÏÑÔŒÙÛÜÿ a-zàâæçéèêëîïñôœùûüÿ'\-]/gi,'');
    else {
	cityName = scheduleCitiesInseeNameLatLon[cityCode][0];
	lat = scheduleCitiesInseeNameLatLon[cityCode][1];
	lon = scheduleCitiesInseeNameLatLon[cityCode][2];
    }
    var recorded = false;
    ul.find ('li[insee="'+cityCode+'"][lat="'+lat+'"][lon="'+lon+'"]').each (function (index) {
	recorded = true;
	scheduleSetSelectCity (jQuery (this));
    });
    if (recorded)
	return;
    var lonLat =
	(lat && lon) ?
	' lat="'+lat+'" lon="'+lon+'"' :
	' class="unknown"';
    ul.append ('<li insee="'+cityCode+'"'+lonLat+' onclick="scheduleSelectCity (this)">'+
	       '<img class="checked" src="'+scheduleEmptyUrl+'" onclick="scheduleRemoveCity (this)"/>'+
	       '<span class="addresse"></span> '+
	       '<span class="city">'+cityName+'</span></li>');
    scheduleSetSelectCity (ul.find ("li").last ());
}

// ========================================
/* Sort UL by city name (XXX what about addresse ?) */
function scheduleSortUlCities (ul) {
    ul.children ('li').sort (function (a, b) {
	var upa = jQuery (a).find ('span.city').text ().toUpperCase ();
	var upb = jQuery (b).find ('span.city').text ().toUpperCase ();
	return (upa < upb) ? -1 : (upa > upb) ? 1 : 0;
    }).appendTo (ul[0]);
}

function scheduleAjaxPOILine (action) {
    if (!scheduleSelectedCity)
	// XXX dire quelque chose
	return;
    var insee = scheduleSelectedCity.attr ('insee');
    var lat = scheduleSelectedCity.attr ('lat');
    var lon = scheduleSelectedCity.attr ('lon');
    var addr = scheduleSelectedCity.find ('span.addresse').html ().replace (/<br>/gi, '~br~');
    line = new Array ();
    line.push (insee);
    line.push (lat);
    line.push (lon);
    line.push (addr);
    jQuery.ajax ({
	type: "POST",
	url: DOKU_BASE+"lib/plugins/schedule/ajaxPOI.php",
	cache: true,
	async: true,
        data: { action: action, line: line.join ('|') },
	success: function (response) { alert (response); }
    });
}

function scheduleAddInsee () {
    scheduleAjaxPOILine ("add");
}
function scheduleRemoveInsee () {
    scheduleAjaxPOILine ("remove");
}
function scheduleTestInsee () {
}

function scheduleSetSelectCity (li) {
    scheduleSelectedCity = li;
    // change autocomplete
    var cityCode = scheduleSelectedCity.attr ('insee');
    var form = scheduleSelectedCity.closest ('.scheduleCitiesForm');
    var input = form.find ('input[name="addr"]');
    input. val (scheduleSelectedCity.find ('span.addresse').html ().replace (/<br>/gi, '~br~'));
    jQuery.ajax ({
	type: "POST",
	url: DOKU_BASE+"lib/plugins/schedule/ajaxPOI.php",
	cache: true,
	async: true,
        data: { action: "list", insee: cityCode },
	success: function (response) {
	    var db = jQuery.parseJSON (response);
	    source = new Array ();
	    for (var i = 0; i < db.length; i++) {
		var vals = db[i].split ("|"); 
		source.push ("("+vals[0]+"|"+vals[1]+") "+vals[2]);
	    }
	    input.autocomplete ({ source: source });
	}
    });
}

function scheduleFocusCity () {
    var form = scheduleSelectedCity.closest ('.scheduleCitiesForm');
    // focus on map
    form.find ('.scheduleMap').each (function () {
	var scheduleMap = scheduleMapList [jQuery (this).attr ('id')];
	scheduleMap.map.
	    setCenter (new OpenLayers.
		       LonLat (scheduleSelectedCity.attr ('lon'),
			       scheduleSelectedCity.attr ('lat')).transform (
				   new OpenLayers.Projection ('EPSG:4326'),
				   new OpenLayers.Projection ('EPSG:900913')),
		       Math.max (scheduleMap.map.getZoom (), 16));
    });    
}

// ========================================
/* User LI selection and focus on map */
function scheduleSelectCity (item) {
    scheduleSetSelectCity (jQuery (item).closest ('li'));
    scheduleFocusCity ();
}

// ========================================
/* User remove LI */
function scheduleRemoveCity (item) {
    var li = jQuery (item).closest ('li');
    var ul = li.closest ('ul');
    li.remove ();
    scheduleUpdateUlCity (ul);
}

function scheduleUpdateUlCity (ul) {
    ul.closest ('.scheduleCitiesForm'). find ('.scheduleMap').each (function () {
	var scheduleMap = scheduleMapList [jQuery (this).attr ('id')];
	scheduleAddAllCityFromUl (scheduleMap, ul);
	scheduleCenterMap (scheduleMap);
    });
    scheduleCheckInputs ();
}

// ========================================
/* Find all initial values */
function scheduleInitPOI () {
    jQuery ('.schedulePOI').each (function () {
	var poiDiv = jQuery (this);
	var scheduleMap = scheduleMapList [poiDiv.find ('.scheduleMap').attr ('id')];
	scheduleClearMarkers (scheduleMap);
	poiDiv.find ('span.poiInsee').each (function () {
	    var citiesInitList = jQuery (this).text ().split (',');
	    var regex = new RegExp ("^\\s*\\(\\s*([\\-0-9.]+)\\s*\\|\\s*([\\-0-9.]+)\\s*\\)\\s*$");
	    for (var i = 0; i < citiesInitList.length; i++) {
		var cityCode = citiesInitList[i];
		// XXX si (|) =>
		var match = cityCode.match (regex);
		if (match) {
		    var lat = parseFloat (match [1]).toFixed (4);
		    var lon = parseFloat (match [2]).toFixed (4);
		    scheduleAddLatLonCityMarker (scheduleMap, cityCode, lat, lon, true);
		} else
		    scheduleAddCityMarker (scheduleMap, cityCode);
	    }
	    scheduleCenterMap (scheduleMap);
	});
	poiDiv.find ('ul.poiLatLon').each (function () {
	    jQuery (this).find ('li').each (function (index) {
		var li = jQuery (this);
		scheduleAddLatLonCityMarker (scheduleMap, li.text (),
					     li.attr ('lat'), li.attr ('lon'),
					     true);
	    });
	    scheduleCenterMap (scheduleMap);
	});
    });
    jQuery ('.scheduleCitiesForm').each (function () {
	var scheduleCitiesForm = jQuery (this);
	var scheduleMapDiv = scheduleCitiesForm.find ('.scheduleMap');
	if (scheduleMapDiv[0] === undefined)
	    return;
	var scheduleMap = scheduleMapList [scheduleMapDiv.attr ('id')];
	scheduleAddAllCityFromUl (scheduleMap, jQuery (this).find ('.cities'));
	scheduleCenterMap (scheduleMap);
    });
}

// ========================================
/* Update selected city position */
function scheduleUpdateLonLat (mapId, latLonGPS) {
    if (!scheduleSelectedCity)
	return;
    scheduleSelectedCity.attr ('lat', latLonGPS.lat.toFixed (4));
    scheduleSelectedCity.attr ('lon', latLonGPS.lon.toFixed (4));
    scheduleAddAllCityFromUl (scheduleMapList [mapId], scheduleSelectedCity.closest ('ul'));
}

// ========================================
/* Find all maps */
function scheduleInitMaps () {
    jQuery ('.scheduleMap').each (function () {
	var mapDiv = jQuery (this).find ('div');
	if (mapDiv !== undefined && mapDiv.size () > 0)
	    return;
	var mapId = jQuery (this).attr ('id');
	var map = new OpenLayers.Map (mapId);
	map.addLayer (new OpenLayers.Layer.OSM ());
	var markers = new OpenLayers.Layer.Markers ('Markers');
	map.addLayer (markers);

	var scheduleMap = {
	    mapId: mapId,
	    map: map,
	    bounds: new OpenLayers.Bounds (),
	    markersSize: 0,
	    markers: markers,
	    selectedMarker: null
	};

        map.events.register ("click", map, function (e) {
 	    var position = this.events.getMousePosition (e);
	    var latLon = map.getLonLatFromPixel (position);
	    scheduleUpdateLonLat (mapId, latLon.transform (
		new OpenLayers.Projection ('EPSG:900913'),
		new OpenLayers.Projection ('EPSG:4326')));
	});
	scheduleMapList [mapId] = scheduleMap;
	scheduleCenterMap (scheduleMap);
    });
}

// ========================================
/* Initialisation and attach function */
jQuery (function () {

    jQuery (function () {
	/* create tabs before OpenLayers stuff (important !) */
	jQuery (".scheduleTabForm").tabs ({
	    active: 0
	});
	jQuery ("#scheduleHelp").accordion ({
	    collapsible: true,
	    animated: false,
	    active: false
	});
	jQuery.datepicker.setDefaults ({
	    showOn: "both",
	    buttonImageOnly: true,
	    buttonImage: DOKU_BASE+'lib/plugins/schedule/images/calendar.png',
	    buttonText: "",
	    firstDay: 0
	});
	jQuery.datepicker.formatDate ("yy-mm-dd");
	jQuery (".scheduleTabForm .date").datepicker ();

	if (jQuery ("div.schedule").prev ("div.error").size ())
	    scheduleForceCheckInputs ();
    });


    /* constants */
    scheduleMarkerUrl = DOKU_BASE+'lib/plugins/schedule/images/poi.png';
    scheduleEmptyUrl = DOKU_BASE+'lib/plugins/schedule/images/empty.png';
    OpenLayers.ImgPath = DOKU_BASE+'lib/plugins/schedule/images/ol/';

    iconDim = new OpenLayers.Size (16, 16);
    iconOffset = new OpenLayers.Pixel (-(iconDim.w/2), -iconDim.h);

    scheduleCitiesName = new Array ();
    scheduleCitiesNameInsee = new Array ();
    for (var cityCode in scheduleCitiesInseeNameLatLon) {
	scheduleCitiesName.push (scheduleCitiesInseeNameLatLon[cityCode][0]+" ("+cityCode+")");
	scheduleCitiesNameInsee [scheduleCitiesInseeNameLatLon[cityCode][0].toLowerCase ()] = cityCode;
    }

    /* autocomplete */
    jQuery ('.scheduleCitiesForm input[name="city"]').autocomplete ({
	source: scheduleCitiesName
    });

    /* init maps */
    scheduleInitMaps ();
    scheduleInitPOI ();

    /* forms */
    jQuery ('.scheduleFinalForm').submit (function () {
	var scheduleFinalForm = jQuery (this);
	if (!scheduleForceCheckInputs ())
	    return false;
	var scheduleForm = scheduleFinalForm.closest ('.scheduleTabForm');
	var scheduleCitiesForm = scheduleForm.find ('.scheduleCitiesForm');
	var cities = new Array ();
	var lats = new Array ();
	var lons = new Array ();
	var addrs = new Array ();
	scheduleCitiesForm.find ('li').each (function (index) {
	    var li = jQuery (this);
	    cities.push (li.attr ('insee'));
	    lats.push (li.attr ('lat'));
	    lons.push (li.attr ('lon'));
	    addrs.push (li.find ('span.addresse').html ());
	});
	scheduleFinalForm.append ('<input type="hidden" name="schd[where]" value="'+cities.join (",")+'"/>');
	scheduleFinalForm.append ('<input type="hidden" name="schd[lat]" value="'+lats.join (",")+'"/>');
	scheduleFinalForm.append ('<input type="hidden" name="schd[lon]" value="'+lons.join (",")+'"/>');
	scheduleFinalForm.append ('<input type="hidden" name="schd[addr]" value="'+addrs.join ("|").replace (/<br>/gi, '~br~')+'"/>');

	var scheduleMiscForm = scheduleForm.find ('.scheduleMiscForm');
	scheduleMiscForm.find ('input[type="text"]').each (function (index) {
	    if (this.value)
		scheduleFinalForm.append ('<input type="hidden" name="'+this.name+'" value="'+this.value.replace(/"/gi, "''")+'"/>');
	});
	scheduleMiscForm.find ('select:not([class="members"])').each (function (index) {
	    scheduleFinalForm.append ('<input type="hidden" name="'+this.name+'" value="'+this.value+'"/>');
	});
	var members = new Array ();
	scheduleMiscForm.find ('select[class="members"]').each (function (index) {
	    jQuery (this).find ('option:selected').each (function (index) {
		members.push (jQuery (this).val ());
	    });
	});
	if (members.length > 0)
	    scheduleFinalForm.append ('<input type="hidden" name="schd[member]" value="'+members.join (",")+'"/>');
	scheduleMiscForm.find ('textarea').each (function (index) {
	    var val = jQuery (this).val ().replace(/"/gi, "''");
	    var name = jQuery (this).attr ('name');
	    scheduleFinalForm.append ('<input type="hidden" name="'+name+'" value="'+val+'"/>');
	});
	return true;
    });

    var regex = new RegExp ("^\\s*\\(\\s*([\\-0-9.]+)\\s*\\|\\s*([\\-0-9.]+)\\s*\\)\\s*(.*)$");
    jQuery ('.scheduleCitiesForm input[name="addr"]').keypress (function (e) {
	if (e.which != 13)
	    return;
	if (!scheduleSelectedCity)
	    return;
	var input = jQuery (this);
	var form = input.closest ('.scheduleCitiesForm');
	var addr = input.val ();
	var match = addr.match (regex);
	if (match) {
	    addr = match [3];
	    scheduleSelectedCity.attr ('lat', parseFloat (match [1]).toFixed (4));
	    scheduleSelectedCity.attr ('lon', parseFloat (match [2]).toFixed (4));
	    var scheduleMap = scheduleMapList [form.find ('.scheduleMap').attr ('id')];
	    scheduleAddAllCityFromUl (scheduleMap, scheduleSelectedCity.closest ('ul'));
	    scheduleFocusCity ();
	}
	input.val (addr);
	scheduleSelectedCity.find ('span.addresse').html (addr.replace (/~br~/gi, '<br/>'));
    });

    jQuery ('.scheduleCitiesForm input[name="city"]').keypress (function (e) {
	if (e.which != 13)
	    return;
	var input = jQuery (this);
	var form = input.closest ('.scheduleCitiesForm');
	var cityName = input.val ();
	if (!cityName)
	    return false;
	input.val ("");
	form.find ('input[name="addr"]').val ("");
	var ul = form.find ('ul');
	scheduleAddCityToUl (ul, cityName);
	scheduleSortUlCities (ul);
	scheduleUpdateUlCity (ul);
	return false;
    });

    jQuery ('.scheduleMiscForm input').keypress (function (e) {
	if (e.which != 13)
	    return;
	jQuery (this).closest ('.scheduleTabForm').find ('input[type="submit"]').first ().trigger ('click');
    });
});
// ========================================
